from PySide2 import QtGui
from PySide2.QtGui import Qt
from PySide2.QtWidgets import QWidget, QTableWidgetItem, QAbstractItemView
from axipy import data_manager, SelectionTable, selection_manager, task_manager, AxipyAnyCallableTask

from TableStatistic.ui.add_tools.helper_tab import FildsNumber, createSqlStatistic, tab_sql_toList
from TableStatistic.ui.add_tools.task_statistic import CalcStatistic
from TableStatistic.ui.helperUI import load_ui


class FormStatistic(QWidget):
    def __init__(self,parent=None):
        super().__init__(parent)
        self.ui=load_ui("formStatistic.ui",parent)
        self.ui.chb_only_selection.stateChanged.connect(self.__set_dynamic)
        self.ui.table_statistics.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.__update_list_tables()
        self.ui.pb_run.clicked.connect(self.__run)
        self.__task_statistic=None
        data_manager.added.connect(self.__add_tab)
        data_manager.removed.connect(self.__remove_tab)
    def show(self):
        self.ui.show()
    @property
    def widget(self):
        return self.ui
    def __update_list_tables(self):
        all_tables=data_manager.tables
        tab_sel=data_manager.selection
        self.ui.cmb_tables.clear()
        if tab_sel is not None:
            self.ui.cmb_tables.addItem(tab_sel.name)
        list_query=[]
        list_tab=[]
        for tab in all_tables:
            if isinstance(tab,SelectionTable):
                if tab.name!='Выборка':
                    list_query.append(tab.name)
            else:
                list_tab.append(tab.name)
        if len(list_query)>0:
            for item in list_query:
                self.ui.cmb_tables.addItem(item)
        if len(list_tab)>0:
            for item in list_tab:
                self.ui.cmb_tables.addItem(item)
        self.ui.cmb_tables.setCurrentIndex(0)
    def __set_dynamic(self):
        if self.ui.chb_only_selection.isChecked():
            selection_manager.changed.connect(self.__run_dynamic)
            self.ui.pb_run.setEnabled(False)
            #print("Set False 52")
        else:
            selection_manager.changed.disconnect(self.__run_dynamic)
            self.ui.pb_run.setEnabled(True)
            self.__update_list_tables()
    def __run_dynamic(self,tab_processing=None):


        #self.__run(data_manager.selection)
        if self.__task_statistic is not None:
            return
        if tab_processing is None and data_manager.selection is None:
            return
        tab_pro=None
        if tab_processing is None:
            tab_pro=data_manager.selection
            print(tab_pro.name)
        else:
            tab_pro=tab_processing
        self.__task_statistic=CalcStatistic(tab_pro)
        self.__task_statistic.end_task.connect(self.__finish_task)
        self.__task = AxipyAnyCallableTask(self.__task_statistic.run, self)
        self.__task.with_handler(True)
        task_manager.start_task(self.__task)
    def __run(self,tab_sel=None):
        '''
        if tab_sel is None:
            name_tab=self.ui.cmb_tables.currentText()
            tab_processing = data_manager.find(name_tab)
        else:
            name_tab=tab_sel.name
            tab_processing=tab_sel
        '''
        self.ui.pb_run.setEnabled(False)

        name_tab = self.ui.cmb_tables.currentText()
        tab_processing = data_manager.find(name_tab)
        #self.__run_dynamic(tab_processing)
        #return
        list_fields=FildsNumber(tab_processing)
        sql = createSqlStatistic(list_fields)
        sql_run = sql + " From " + tab_processing.name
        tab_sql = data_manager.query_hidden(sql_run)
        statistic_info = tab_sql_toList(tab_sql, list_fields)
        tab_sql.close()
        self.ui.table_statistics.setRowCount(0)
        count_val=statistic_info['count']
        self.ui.ln_count.setText(str(count_val))
        fields_statistic=statistic_info['fields']
        count_fields=len(fields_statistic)
        self.ui.table_statistics.setRowCount(count_fields)
        for i_row,f_stat in enumerate(fields_statistic):
            self.ui.table_statistics.setItem(i_row, 0, QTableWidgetItem(f_stat['name']))

            stat_info=f_stat['info']
            #item = QTableWidgetItem(str(stat_info['min'])).setTextAlignment(Qt.AlignRight)
            item = QTableWidgetItem(str(stat_info['min']))
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            self.ui.table_statistics.setItem(i_row, 1, item)
            item = QTableWidgetItem(str(stat_info['max']))
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            self.ui.table_statistics.setItem(i_row, 2, item)
            item = QTableWidgetItem(str(stat_info['sum']))
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            self.ui.table_statistics.setItem(i_row, 3, item)
            item = QTableWidgetItem(str(stat_info['avg']))
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            self.ui.table_statistics.setItem(i_row, 4, item)
            item = QTableWidgetItem(str(stat_info['std']))
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            self.ui.table_statistics.setItem(i_row, 5, item)
            if stat_info['std_e'] is not None:
                item = QTableWidgetItem(str(stat_info['std_e']))
                item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
                self.ui.table_statistics.setItem(i_row, 6, item)
        #print("end")
        self.ui.pb_run.setEnabled(True)
        #print("end 1")
    def __finish_task(self):
        #self.ui.pb_run.setEnabled(True)
        if self.__task_statistic is None:
            return
        statistic_info=self.__task_statistic.result
        self.ui.table_statistics.setRowCount(0)
        if statistic_info is None:
            return
        self.ui.table_statistics.setRowCount(0)
        count_val = statistic_info['count']
        self.ui.ln_count.setText(str(count_val))
        fields_statistic = statistic_info['fields']
        count_fields = len(fields_statistic)
        self.ui.table_statistics.setRowCount(count_fields)
        for i_row, f_stat in enumerate(fields_statistic):
            self.ui.table_statistics.setItem(i_row, 0, QTableWidgetItem(f_stat['name']))

            stat_info = f_stat['info']
            # item = QTableWidgetItem(str(stat_info['min'])).setTextAlignment(Qt.AlignRight)
            item = QTableWidgetItem(str(stat_info['min']))
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            self.ui.table_statistics.setItem(i_row, 1, item)
            item = QTableWidgetItem(str(stat_info['max']))
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            self.ui.table_statistics.setItem(i_row, 2, item)
            item = QTableWidgetItem(str(stat_info['sum']))
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            self.ui.table_statistics.setItem(i_row, 3, item)
            item = QTableWidgetItem(str(stat_info['avg']))
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            self.ui.table_statistics.setItem(i_row, 4, item)
            item = QTableWidgetItem(str(stat_info['std']))
            item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
            self.ui.table_statistics.setItem(i_row, 5, item)
            if stat_info['std_e'] is not None:
                item = QTableWidgetItem(str(stat_info['std_e']))
                item.setTextAlignment(Qt.AlignRight | Qt.AlignVCenter)
                self.ui.table_statistics.setItem(i_row, 6, item)
        self.__task_statistic.end_task.disconnect(self.__finish_task)
        self.__task_statistic=None
        self.__task=None

    def __add_tab(self,name_tab):
        if not self.ui.pb_run.isEnabled():
            return
        if name_tab=="Выборка":
            self.ui.cmb_tables.insertItem(0,name_tab)
            return
        tab_add=data_manager.find(name_tab)
        if isinstance(tab_add,SelectionTable):
            ''' Добавляется запрос'''
            last_index_query=self.__find_index_lastTabQuery()
            if last_index_query==-1:
                self.ui.cmb_tables.addItem(name_tab)
                return
            self.ui.cmb_tables.insertItem(last_index_query+1, name_tab)
            return
        self.ui.cmb_tables.addItem(name_tab)

    def __remove_tab(self,name_tab):
        id=self.ui.cmb_tables.findText(name_tab)
        self.ui.cmb_tables.removeItem(id)
        if self.ui.cmb_tables.count()==0:
            self.ui.pb_run.setEnabled(False)
            #print("set False 194")
    def __find_index_lastTabQuery(self):
        index=-1
        count=self.ui.cmb_tables.count()
        for i in range(count):
            name_tab=self.ui.cmb_tables.itemText(i)
            tab_temp=data_manager.find(name_tab)
            if isinstance(tab_temp,SelectionTable):
                index=i
                continue
            if index>=0:
                return index-1
        return index



